from pathlib import Path

import click
import requests
from loguru import logger

from ts_bolt.datasets.collections import DownloaderDataset, RawFileDataset
from ts_bolt.datasets.collections import collections as dataset_collections


def download_binary_request(dataset: RawFileDataset, local_file: Path) -> None:
    """Download remote content in a binary format

    :param dataset: a RawFileDataset definition
    :param local_file: where to write the content to
    """

    r = requests.get(dataset.remote)
    if r.status_code != 200:
        logger.error(f"Can not download {dataset}")
    else:
        with open(local_file, "wb") as f:
            for chunk in r.iter_content(chunk_size=128):
                f.write(chunk)


@click.group(invoke_without_command=True)
@click.pass_context
def bolt(ctx) -> None:  # type: ignore[no-untyped-def]
    if ctx.invoked_subcommand is None:
        click.echo("Use bolt --help for help.")
    else:
        pass


@bolt.command()
@click.option(
    "--name",
    default=None,
    type=click.Choice(list(dataset_collections.keys())),  # type: ignore[has-type]
    help="name of dataset to be check",
    required=False,
)
def list(name: str) -> None:
    """
    :param name: name of datasets to list
    """
    if name is None:
        click.echo_via_pager("\n".join(f"{d}" for d in dataset_collections))
    else:
        click.secho(f"{name} definition: {dataset_collections[name]}")


@bolt.command()
@click.option(
    "--name",
    type=click.Choice(dataset_collections.keys()),
    help="name of dataset to be downloaded",
    required=True,
)
@click.option(
    "--target",
    type=click.Path(),
    callback=lambda _, __, value: Path(value),
    help="where to save the dataset",
    required=True,
)
def download(name: str, target: Path) -> None:
    dataset = dataset_collections[name]
    if not target.exists():
        target.mkdir(parents=True, exist_ok=False)
    local_file = target / dataset.file_name  # type: ignore[attr-defined]

    if local_file.exists():
        logger.warning(
            f"file already exists in {local_file}; please remove if redownload is needed"
        )
    else:
        if isinstance(dataset, RawFileDataset):
            download_binary_request(dataset, local_file)
        elif isinstance(dataset, DownloaderDataset):
            dataset.downloader(local_file)


if __name__ == "__main__":
    pass



from functools import cached_property
from typing import Tuple

import lightning as L
import numpy as np
import pandas as pd
import torch
from loguru import logger
from torch.utils.data import DataLoader, Dataset


class DataFrameDataset(Dataset):
    """A dataset from a pandas dataframe.

    For a given pandas dataframe, this generates a pytorch
    compatible dataset by sliding in time dimension.

    ```python
    ds = DataFrameDataset(
        dataframe=df, history_length=10, horizon=2
    )
    ```

    :param dataframe: input dataframe with a DatetimeIndex.
    :param history_length: length of input X in time dimension
        in the final Dataset class.
    :param horizon: number of steps to be forecasted.
    :param gap: gap between input history and prediction
    """

    def __init__(
        self, dataframe: pd.DataFrame, history_length: int, horizon: int, gap: int = 0
    ):
        super().__init__()
        self.dataframe = dataframe
        self.history_length = history_length
        self.horzion = horizon
        self.gap = gap
        self.dataframe_rows = len(self.dataframe)
        self.length = (
            self.dataframe_rows - self.history_length - self.horzion - self.gap + 1
        )

    def moving_slicing(self, idx: int, gap: int = 0) -> Tuple[np.ndarray, np.ndarray]:
        x, y = (
            self.dataframe[idx : self.history_length + idx].values,
            self.dataframe[
                self.history_length
                + idx
                + gap : self.history_length
                + self.horzion
                + idx
                + gap
            ].values,
        )
        return x, y

    def _validate_dataframe(self) -> None:
        """Validate the input dataframe.

        - We require the dataframe index to be DatetimeIndex.
        - This dataset is null aversion.
        - Dataframe index should be sorted.
        """

        if not isinstance(
            self.dataframe.index, pd.core.indexes.datetimes.DatetimeIndex
        ):
            raise TypeError(
                "Type of the dataframe index is not DatetimeIndex"
                f": {type(self.dataframe.index)}"
            )

        has_na = self.dataframe.isnull().values.any()

        if has_na:
            logger.warning("Dataframe has null")

        has_index_sorted = self.dataframe.index.equals(
            self.dataframe.index.sort_values()
        )

        if not has_index_sorted:
            logger.warning("Dataframe index is not sorted")

    def __getitem__(self, idx: int) -> Tuple[np.ndarray, np.ndarray]:
        if isinstance(idx, slice):
            if (idx.start < 0) or (idx.stop >= self.length):
                raise IndexError(f"Slice out of range: {idx}")
            step = idx.step if idx.step is not None else 1
            return [
                self.moving_slicing(i, self.gap)
                for i in range(idx.start, idx.stop, step)
            ]
        else:
            if idx >= self.length:
                raise IndexError("End of dataset")
            return self.moving_slicing(idx, self.gap)

    def __len__(self) -> int:
        return self.length


class DataFrameDataModule(L.LightningDataModule):
    def __init__(
        self,
        history_length: int,
        horizon: int,
        dataframe: pd.DataFrame,
        gap: int = 0,
        test_fraction: float = 0.3,
        val_fraction: float = 0.1,
        batch_size: int = 32,
        num_workers: int = 0,
    ):
        super().__init__()
        self.history_length = history_length
        self.horizon = horizon
        self.batch_size = batch_size
        self.dataframe = dataframe
        self.gap = gap
        self.test_fraction = test_fraction
        self.val_fraction = val_fraction
        self.num_workers = num_workers

        self.train_dataset, self.val_dataset = self.split_train_val(
            self.train_val_dataset
        )

    @cached_property
    def df_length(self) -> int:
        return len(self.dataframe)

    @cached_property
    def df_test_length(self) -> int:
        return int(self.df_length * self.test_fraction)

    @cached_property
    def df_train_val_length(self) -> int:
        return self.df_length - self.df_test_length

    @cached_property
    def train_val_dataframe(self) -> pd.DataFrame:
        return self.dataframe.iloc[: self.df_train_val_length]

    @cached_property
    def test_dataframe(self) -> pd.DataFrame:
        return self.dataframe.iloc[self.df_train_val_length :]

    @cached_property
    def train_val_dataset(self) -> DataFrameDataset:
        return DataFrameDataset(
            dataframe=self.train_val_dataframe,
            history_length=self.history_length,
            horizon=self.horizon,
            gap=self.gap,
        )

    @cached_property
    def test_dataset(self) -> DataFrameDataset:
        return DataFrameDataset(
            dataframe=self.test_dataframe,
            history_length=self.history_length,
            horizon=self.horizon,
            gap=self.gap,
        )

    def split_train_val(self, dataset: Dataset) -> list[torch.Tensor]:
        return torch.utils.data.random_split(
            dataset, [1 - self.val_fraction, self.val_fraction]
        )

    def train_dataloader(self) -> DataLoader:
        return DataLoader(
            dataset=self.train_dataset,
            batch_size=self.batch_size,
            shuffle=True,
            num_workers=self.num_workers,
            persistent_workers=True if self.num_workers > 0 else False,
        )

    def test_dataloader(self) -> DataLoader:
        return DataLoader(
            dataset=self.test_dataset,
            batch_size=self.batch_size,
            shuffle=False,
            num_workers=self.num_workers,
        )

    def val_dataloader(self) -> DataLoader:
        return DataLoader(
            dataset=self.val_dataset,
            batch_size=self.batch_size,
            shuffle=False,
            num_workers=self.num_workers,
            persistent_workers=True if self.num_workers > 0 else False,
        )

    def predict_dataloader(self) -> DataLoader:
        return DataLoader(
            dataset=self.test_dataset, batch_size=len(self.test_dataset), shuffle=False
        )



from pydantic import BaseModel, HttpUrl

from ts_bolt.datasets.downloaders import BaseDownloader, ECBExchangeRateDownloader


class NamedDatasets(BaseModel):
    """
    NamedDatasets is a collection of datasets
    that are used in the ts_bolt library.
    """

    lai_exchange_rate: str = "lai_exchange_rate"
    lai_electricity: str = "lai_electricity"
    lai_solar_al: str = "lai_solar_al"
    lai_traffic: str = "lai_traffic"
    ecb_exchange_rate: str = "ecb_exchange_rate"
    mbohlkeschneider_electricity_nips: str = "mbohlkeschneider_electricity_nips"
    mbohlkeschneider_exchange_rate_nips: str = "mbohlkeschneider_exchange_rate_nips"
    mbohlkeschneider_solar_nips: str = "mbohlkeschneider_solar_nips"
    mbohlkeschneider_wiki_rolling_nips: str = "mbohlkeschneider_wiki_rolling_nips"
    mbohlkeschneider_traffic_nips: str = "mbohlkeschneider_traffic_nips"
    mbohlkeschneider_taxi_30min: str = "mbohlkeschneider_taxi_30min"


class RawFileDataset(BaseModel):
    """A model defining raw file datasets."""

    name: str
    remote: str
    documentation: str
    file_name: str
    description: str


class DownloaderDataset(BaseModel):
    """Dataset that is downloaded by a downloader function."""

    name: str
    downloader: BaseDownloader
    documentation: str
    file_name: str
    description: str


collections = {
    NamedDatasets.lai_exchange_rate: RawFileDataset(
        name=NamedDatasets.lai_exchange_rate,
        remote=(
            "https://github.com/laiguokun/"
            "multivariate-time-series-data/"
            "raw/master/exchange_rate/exchange_rate.txt.gz"
        ),
        documentation="https://github.com/laiguokun/multivariate-time-series-data",
        file_name=f"{NamedDatasets.lai_exchange_rate}.txt.gz",
        description="exchange rates in tabular format",
    ),
    NamedDatasets.lai_electricity: RawFileDataset(
        name=NamedDatasets.lai_electricity,
        remote=(
            "https://github.com/laiguokun/"
            "multivariate-time-series-data/raw/master/"
            "electricity/electricity.txt.gz"
        ),
        documentation=("https://github.com/laiguokun/" "multivariate-time-series-data"),
        file_name="{NamedDatasets.lai_electricity}.txt.gz",
        description="UCI electricity data in tabular format",
    ),
    NamedDatasets.lai_solar_al: RawFileDataset(
        name=NamedDatasets.lai_solar_al,
        remote=(
            "https://github.com/laiguokun/"
            "multivariate-time-series-data/raw/master/"
            "solar-energy/solar_AL.txt.gz"
        ),
        documentation=("https://github.com/laiguokun/" "multivariate-time-series-data"),
        file_name=f"{NamedDatasets.lai_solar_al}.txt.gz",
        description="solar AL in tabular format",
    ),
    NamedDatasets.lai_traffic: RawFileDataset(
        name=NamedDatasets.lai_traffic,
        remote=(
            "https://github.com/laiguokun/"
            "multivariate-time-series-data/raw/master/"
            "traffic/traffic.txt.gz"
        ),
        documentation=("https://github.com/laiguokun/" "multivariate-time-series-data"),
        file_name=f"{NamedDatasets.lai_traffic}.txt.gz",
        description="traffic dataset in tabular format",
    ),
    NamedDatasets.ecb_exchange_rate: DownloaderDataset(
        name=NamedDatasets.ecb_exchange_rate,
        downloader=ECBExchangeRateDownloader(),
        documentation=(
            "https://www.ecb.europa.eu/stats/"
            "policy_and_exchange_rates/"
            "euro_reference_exchange_rates/html/index.en.html"
        ),
        file_name=f"{NamedDatasets.ecb_exchange_rate}.csv",
        description="ecb exchange rate downloaded live in csv format",
    ),
    NamedDatasets.mbohlkeschneider_electricity_nips: RawFileDataset(
        name=NamedDatasets.mbohlkeschneider_electricity_nips,
        remote=(
            "https://github.com/mbohlkeschneider/gluon-ts/"
            "raw/mv_release/datasets/electricity_nips.tar.gz"
        ),
        documentation="https://arxiv.org/abs/2101.12072",
        file_name=f"{NamedDatasets.mbohlkeschneider_electricity_nips}.tar.gz",
        description="gluonts dataset format",
    ),
    NamedDatasets.mbohlkeschneider_exchange_rate_nips: RawFileDataset(
        name=NamedDatasets.mbohlkeschneider_exchange_rate_nips,
        remote=(
            "https://github.com/mbohlkeschneider/gluon-ts/"
            "raw/mv_release/datasets/exchange_rate_nips.tar.gz"
        ),
        documentation="https://arxiv.org/abs/2101.12072",
        file_name=f"{NamedDatasets.mbohlkeschneider_exchange_rate_nips}.tar.gz",
        description="gluonts dataset format",
    ),
    NamedDatasets.mbohlkeschneider_solar_nips: RawFileDataset(
        name=NamedDatasets.mbohlkeschneider_solar_nips,
        remote="https://github.com/mbohlkeschneider/gluon-ts/raw/mv_release/datasets/solar_nips.tar.gz",
        documentation="https://arxiv.org/abs/2101.12072",
        file_name=f"{NamedDatasets.mbohlkeschneider_solar_nips}.tar.gz",
        description="gluonts dataset format",
    ),
    NamedDatasets.mbohlkeschneider_wiki_rolling_nips: RawFileDataset(
        name=NamedDatasets.mbohlkeschneider_wiki_rolling_nips,
        remote=(
            "https://github.com/mbohlkeschneider/gluon-ts/"
            "raw/mv_release/datasets/wiki-rolling_nips.tar.gz"
        ),
        documentation=(
            "https://github.com/mbohlkeschneider/gluon-ts/" "tree/mv_release/datasets"
        ),
        file_name=f"{NamedDatasets.mbohlkeschneider_wiki_rolling_nips}.tar.gz",
        description="gluonts dataset format",
    ),
    NamedDatasets.mbohlkeschneider_traffic_nips: RawFileDataset(
        name=NamedDatasets.mbohlkeschneider_traffic_nips,
        remote=(
            "https://github.com/mbohlkeschneider/gluon-ts/"
            "raw/mv_release/datasets/traffic_nips.tar.gz"
        ),
        documentation=(
            "https://github.com/mbohlkeschneider/gluon-ts/" "tree/mv_release/datasets"
        ),
        file_name=f"{NamedDatasets.mbohlkeschneider_traffic_nips}.tar.gz",
        description="gluonts dataset format",
    ),
    NamedDatasets.mbohlkeschneider_taxi_30min: RawFileDataset(
        name=NamedDatasets.mbohlkeschneider_taxi_30min,
        remote=(
            "https://github.com/mbohlkeschneider/gluon-ts/"
            "raw/mv_release/datasets/taxi_30min.tar.gz"
        ),
        documentation=(
            "https://github.com/mbohlkeschneider/gluon-ts/" "tree/mv_release/datasets"
        ),
        file_name=f"{NamedDatasets.mbohlkeschneider_taxi_30min}.tar.gz",
        description="gluonts dataset format",
    ),
}



from ts_bolt.datasets.downloaders.base import BaseDownloader
from ts_bolt.datasets.downloaders.ecb_exchange_rate import ECBExchangeRateDownloader

__all__ = ["BaseDownloader", "ECBExchangeRateDownloader"]



from abc import ABC, abstractmethod
from typing import Any


class BaseDownloader(ABC):
    """Base class for a data file downloader."""

    @abstractmethod
    def __init__(self, *args: Any, **kwargs: Any):
        pass

    @abstractmethod
    def __call__(self, *args: Any, **kwargs: Any) -> Any:
        pass



from dataclasses import dataclass
from typing import Any, Callable, Dict, Iterable, List, NamedTuple, Optional

import pytorch_lightning as pl
from gluonts.dataset import Dataset
from gluonts.dataset.field_names import FieldName
from gluonts.torch.batchify import batchify
from gluonts.transform import (
    AddObservedValuesIndicator,
    ExpectedNumInstanceSampler,
    InstanceSplitter,
    Transformation,
)
from torch.utils.data import DataLoader, Dataset, IterableDataset


class GluonTSTransformsDefault(Transformation):
    """Default transforms of a gluonts dataset

    ```python
    gluonts_transform = GluonTSTransformsDefault(
        context_length=10,
        prediction_length=5,
    )
    ```

    :param context_length: the length of history input
    :param prediction_length: the length to be forecasted
    """

    def __init__(self, context_length: int, prediction_length: int):
        self.context_length = context_length
        self.prediction_length = prediction_length

    def __call__(self, data_it: Iterable[Dict[str, Any]], is_train: bool):
        mask_unobserved = AddObservedValuesIndicator(
            target_field=FieldName.TARGET,
            output_field=FieldName.OBSERVED_VALUES,
        )

        training_splitter = InstanceSplitter(
            target_field=FieldName.TARGET,
            is_pad_field=FieldName.IS_PAD,
            start_field=FieldName.START,
            forecast_start_field=FieldName.FORECAST_START,
            instance_sampler=ExpectedNumInstanceSampler(
                num_instances=1,
                min_future=self.prediction_length,
            ),
            past_length=self.context_length,
            future_length=self.prediction_length,
            time_series_fields=[FieldName.OBSERVED_VALUES],
        )

        transforms = mask_unobserved + training_splitter

        return transforms(data_it=data_it, is_train=is_train)


class GluonTSDataset(IterableDataset):
    """An iter style dataset built from a gluonts dataset

    ```python
    from gluonts.dataset.repository.datasets import get_dataset

    gluonts_datasets = get_dataset("electricity")

    dataset = GluonTSDataset(
        dataset = gluonts_datasets.train,
        is_train = True
    )
    ```

    :param dataset: gluonts dataset, e.g., TrainDatasets
    :param is_train: whether the dataset is for training
    :param transform: transformations on dataset, e.g., gluonts.transform.InstanceSplitter
    """

    def __init__(
        self,
        dataset: Dataset,
        is_train: bool,
        transform: Optional[Callable] = None,
        metadata: Optional[Dict[Any, Any]] = None,
    ):
        self.metadata = metadata
        self.dataset = dataset
        self.is_train = is_train

        self.transform = transform
        self.transformed_dataset = self._transform_dataset()

    def __iter__(self):
        for d in self.transformed_dataset:
            yield d

    def _transform_dataset(self) -> List[Dict[str, Any]]:

        if self.transform:
            dataset = self.transform(self.dataset, is_train=self.is_train)
        else:
            dataset = self.dataset

        return dataset


@dataclass
class GluonTSDataLoaderConfig:
    """Configs for dataloaders from a gluonts dataset


    ```python
    dl_config = GluonTSDataLoaderConfig(
        batch_size=2,
        transform=None,
        collate_fn=None,
    )
    ```

    :param batch_size: batch size for the PyTorch DataLoader
    :param transform: transforms of the PyTorch DataLoader, e.g., GluonTSTransformsDefault.
    :param collate_fn: collate_fn of the PyTorch DataLoader, e.g., gluonts.torch.batchify.batchify
    """

    batch_size: int
    transform: Optional[Callable]
    collate_fn: Optional[Callable]

    def __post_init__(self):
        if self.collate_fn is None:
            self.collate_fn = batchify


class GluonTSDataModule(pl.LightningDataModule):
    """LightningDataModule from a gluonts dataset.


    ```python
    from gluonts.dataset.repository.datasets import get_dataset

    gluonts_datasets = get_dataset("electricity")

    train_dl_config = GluonTSDataLoaderConfig(
        batch_size=2,
        transform=None,
        collate_fn=None,
    )
    test_dl_config = GluonTSDataLoaderConfig(
        batch_size=10,
        transform=None,
        collate_fn=None,
    )

    dm = GluonTSDataModule(
        train_dataset = gluonts_datasets.train,
        test_dataset = gluonts_datasets.test,
        train_dataloader_config = train_dl_config,
        test_dataloader_config = test_dl_config,
    )
    ```

    :param train_dataset: gluonts Dataset for training
    :param train_dataset: gluonts Dataset for testing
    :param train_dataloader_config: config for train DataLoader
    :param test_dataloader_config: config for the test DataLoader
    """

    def __init__(
        self,
        train_dataset: Dataset,
        test_dataset: Dataset,
        train_dataloader_config: GluonTSDataLoaderConfig,
        test_dataloader_config: GluonTSDataLoaderConfig,
    ):
        super().__init__()
        self.train_dataset = train_dataset
        self.test_dataset = test_dataset
        self.train_dataloader_config = train_dataloader_config
        self.test_dataloader_config = test_dataloader_config

    def train_dataloader(self):
        return DataLoader(
            dataset=GluonTSDataset(
                dataset=self.train_dataset,
                is_train=True,
                transform=self.train_dataloader_config.transform,
            ),
            batch_size=self.train_dataloader_config.batch_size,
            collate_fn=self.train_dataloader_config.collate_fn,
        )

    def test_dataloader(self):
        return DataLoader(
            dataset=GluonTSDataset(
                dataset=self.test_dataset,
                is_train=False,
                transform=self.test_dataloader_config.transform,
            ),
            batch_size=self.test_dataloader_config.batch_size,
            collate_fn=self.test_dataloader_config.collate_fn,
        )



from typing import List, Sequence, Tuple

import lightning.pytorch as L
import torch


class LastObservationForecaster(L.LightningModule):
    """Spits out the forecasts using the last observation.

    :param horizon: horizon of the forecast.
    """

    def __init__(self, horizon: int):
        super().__init__()
        self.horizon = horizon

    def _last_observation(self, x: torch.Tensor) -> torch.Tensor:
        return x[..., -1:, :]

    def predict_step(
        self, batch: Sequence[torch.Tensor], batch_idx: int
    ) -> Tuple[torch.Tensor, torch.Tensor]:
        x, y = batch

        y_hat = self._last_observation(x)

        y_hat = y_hat.repeat(1, self.horizon, 1)

        return x.squeeze(-1), y_hat.squeeze(-1)

    def forward(self, x: torch.Tensor) -> Tuple[torch.Tensor, torch.Tensor]:
        x = x.type(self.dtype)
        return (
            x.squeeze(-1),
            self._last_observation(x).repeat(1, self.horizon, 1).squeeze(-1),
        )



from typing import Dict, List, Sequence, Tuple

import matplotlib as mpl
import numpy as np
import pandas as pd
import torch
from torch.utils.data import DataLoader
from torchmetrics import MetricCollection
from torchmetrics.regression import (
    MeanAbsoluteError,
    MeanAbsolutePercentageError,
    MeanSquaredError,
    SymmetricMeanAbsolutePercentageError,
)


class Evaluator:
    """Evaluate the predictions

    :param step: which prediction step to be evaluated.
    :param gap: gap between input history and target/prediction.
    """

    def __init__(self, step: int = 0, gap: int = 0):
        self.step = step
        self.gap = gap

    @staticmethod
    def get_one_history(
        predictions: Sequence[Sequence], idx: int, batch_idx: int = 0
    ) -> torch.Tensor:
        return predictions[batch_idx][0][idx, ...]

    @staticmethod
    def get_one_pred(predictions: List, idx: int, batch_idx: int = 0) -> torch.Tensor:
        return predictions[batch_idx][1][idx, ...]

    @staticmethod
    def get_y(predictions: List, step: int) -> List[torch.Tensor]:
        return [i[1][..., step] for i in predictions]

    def y(self, predictions: List, batch_idx: int = 0) -> torch.Tensor:
        return self.get_y(predictions, self.step)[batch_idx].detach()

    @staticmethod
    def get_y_true(dataloader: DataLoader, step: int) -> list[torch.Tensor]:
        return [i[1].squeeze(-1)[..., step] for i in dataloader]

    def y_true(self, dataloader: DataLoader, batch_idx: int = 0) -> torch.Tensor:
        return self.get_y_true(dataloader, step=self.step)[batch_idx].detach()

    def get_one_sample(
        self, predictions: List, idx: int, batch_idx: int = 0
    ) -> Tuple[torch.Tensor, torch.Tensor]:
        return (
            self.get_one_history(predictions, idx, batch_idx),
            self.get_one_pred(predictions, idx, batch_idx),
        )

    def plot_one_sample(
        self, ax: mpl.axes.Axes, predictions: List, idx: int, batch_idx: int = 0
    ) -> None:
        history, pred = self.get_one_sample(predictions, idx, batch_idx)

        x_raw = np.arange(len(history) + len(pred) + self.gap)
        x_history = x_raw[: len(history)]
        x_pred = x_raw[len(history) + self.gap :]
        x = np.concatenate([x_history, x_pred])

        y = np.concatenate([history, pred])

        ax.plot(x, y, marker=".", label=f"input ({idx})")

        ax.axvspan(x_pred[0], x_pred[-1], color="orange", alpha=0.1)

    @property
    def metric_collection(self) -> MetricCollection:
        return MetricCollection(
            MeanAbsoluteError(),
            MeanAbsolutePercentageError(),
            MeanSquaredError(),
            SymmetricMeanAbsolutePercentageError(),
        )

    @staticmethod
    def metric_dataframe(metrics: Dict) -> pd.DataFrame:
        return pd.DataFrame(
            [{k: float(v) for k, v in metrics.items()}], index=["values"]
        ).T

    def metrics(
        self, predictions: List, dataloader: DataLoader, batch_idx: int = 0
    ) -> pd.DataFrame:
        truths = self.y_true(dataloader)
        preds = self.y(predictions, batch_idx=batch_idx)

        return self.metric_dataframe(self.metric_collection(preds, truths))



from pathlib import Path

import pandas as pd
from loguru import logger

from ts_bolt.datasets.downloaders.base import BaseDownloader


class ECBExchangeRateDownloader(BaseDownloader):
    """Download the ECB Exchange Rate dataset and explore some basic features of the dataset.

    - Dataset Website: https://www.ecb.europa.eu/stats/policy_and_exchange_rates/euro_reference_exchange_rates/html/index.en.html
    - Dataset Download Link: https://www.ecb.europa.eu/stats/eurofxref/eurofxref-hist.zip
    """

    def __init__(self) -> None:
        self.url = "https://www.ecb.europa.eu" "/stats/eurofxref/eurofxref-hist.zip"

    def __call__(self, target: Path) -> None:
        """Download the ECB Exchange Rate dataset and explore some basic features of the dataset.
        - Dataset Website: https://www.ecb.europa.eu/stats/policy_and_exchange_rates/euro_reference_exchange_rates/html/index.en.html
        - Dataset Download Link: https://www.ecb.europa.eu/stats/eurofxref/eurofxref-hist.zip
        """
        if target.exists():
            logger.warning(f"dataset exists in {target}, download skipped")
        else:
            df = pd.read_csv(
                self.url,
                compression="zip",
            )

            df["Date"] = pd.to_datetime(df["Date"]).dt.date

            # There are some empty columns.
            columns = [col for col in df.columns if not df[col].isna().all()]

            df = df[columns]

            df.to_csv(target, index=False)
